#include "config.h"

#include "mplaylib.h"


#include "bswap.h"
#include "subopt-helper.h"
#include "libaf/af_format.h"
#include "audio_out.h"
#include "audio_out_internal.h"
#include "mp_msg.h"
#include "help_mp.h"
#include "ao_jz4740_vol.h"

static ao_info_t info = 
{
	"RAW PCM/WAVE file writer audio output",
	"pcm",
	"Atmosfear",
	""
};

LIBAO_EXTERN(ucos)

extern int vo_pts;

//static char *ao_outputfilename = NULL;
//static int ao_pcm_waveheader = 1;
static int fast = 0;

unsigned int mp_init = 0;

#define WAV_ID_RIFF 0x46464952 /* "RIFF" */
#define WAV_ID_WAVE 0x45564157 /* "WAVE" */
#define WAV_ID_FMT  0x20746d66 /* "fmt " */
#define WAV_ID_DATA 0x61746164 /* "data" */
#define WAV_ID_PCM  0x0001

struct WaveHeader
{
	uint32_t riff;
	uint32_t file_length;
	uint32_t wave;
	uint32_t fmt;
	uint32_t fmt_length;
	uint16_t fmt_tag;
	uint16_t channels;
	uint32_t sample_rate;
	uint32_t bytes_per_second;
	uint16_t block_align;
	uint16_t bits;
	uint32_t data;
	uint32_t data_length;
};

/* init with default values */
//static struct WaveHeader wavhdr = {
//	le2me_32(WAV_ID_RIFF),
//        /* same conventions than in sox/wav.c/wavwritehdr() */
//	0, //le2me_32(0x7ffff024),
//	le2me_32(WAV_ID_WAVE),
//	le2me_32(WAV_ID_FMT),
//	le2me_32(16),
//	le2me_16(WAV_ID_PCM),
//	le2me_16(2),
//	le2me_32(44100),
//	le2me_32(192000),
//	le2me_16(4),
//	le2me_16(16),
//	le2me_32(WAV_ID_DATA),
//	0, //le2me_32(0x7ffff000)
//};
//
//static FILE *fp = NULL;

// to set/get/query special features/parameters
static int control(int cmd,void *arg){
    return -1;
}
//from /drv/codec/pcm.h
#define	AFMT_U8			8
#define AFMT_S16_LE		16
#define PCM_SET_SAMPLE_RATE	0
#define PCM_SET_CHANNEL		1
#define PCM_SET_FORMAT		2

#define PCM_GET_SPACE   5

// open & setup audio device
// return: 1=success 0=fail
static int init(int rate,int channels,int format,int flags){
	int m_format;
	
	int bits = 8;
	#if 0
		opt_t subopts[] = {
	  {"waveheader", OPT_ARG_BOOL, &ao_pcm_waveheader, NULL},
	  {"file",       OPT_ARG_MSTRZ, &ao_outputfilename, NULL},
	  {"fast",       OPT_ARG_BOOL, &fast, NULL},
	  {NULL}
	};
	// set defaults
	ao_pcm_waveheader = 1;
	if (subopt_parse(ao_subdevice, subopts) != 0) {
	  return 0;
	}
	#endif
	m_format=AFMT_U8;
	
	switch(format){
	case AF_FORMAT_S8:
		
	case AF_FORMAT_U8:
	    break;
	default:
	    m_format=AFMT_S16_LE;
	    bits = 16;
	    break;
	}
	#define MAX_BUF (0x1000)
	ao_data.outburst = MAX_BUF;
	ao_data.buffersize= 4 * MAX_BUF;
	ao_data.channels=channels;
	ao_data.samplerate=rate;
	ao_data.format=format;
	ao_data.bps=channels*rate*(bits/8);
	
	//wavhdr.channels = le2me_16(ao_data.channels);
	//wavhdr.sample_rate = le2me_32(ao_data.samplerate);
	//wavhdr.bytes_per_second = le2me_32(ao_data.bps);
	//wavhdr.bits = le2me_16(bits);
	//wavhdr.block_align = le2me_16(ao_data.channels * (bits / 8));
	//
	//wavhdr.data_length=le2me_32(0x7ffff000);
	//wavhdr.file_length = wavhdr.data_length + sizeof(wavhdr) - 8;

	
	
	pcm_ioctl(PCM_SET_SAMPLE_RATE,rate);
	//printf("rate = %d\n",rate);
	
	pcm_ioctl(PCM_SET_CHANNEL,channels);
	pcm_ioctl(PCM_SET_FORMAT,m_format);

	SetAudioVolume(replay_fill_all_16_mid);

	mp_init = 0; 
	F("rate = %d channel = %d %d\n",rate,channels,m_format);
	
	
	
	return 1;
}


// close audio device
static void uninit(int immed){

	mp_init = 0;  
	
}
#define PCM_SET_PAUSE		8
#define PCM_SET_PLAY		9
#define PCM_RESET		10

// stop playing and empty buffers (for seeking/pause)
static void reset(void){
	F("1\n");
	
	pcm_ioctl(PCM_RESET, 0);
}

// stop playing, keep buffers (for pause)
static void audio_pause(void)
{
	F("1\n");
	  pcm_ioctl(PCM_SET_PAUSE, 0);
    // for now, just call reset();
    
}

// resume playing, after audio_pause()
static void audio_resume(void)
{
	  F("1\n");
	
	  pcm_ioctl(PCM_SET_PLAY, 0);
}
// return: how many bytes can be played without blocking
static int get_space(void){
		#if 1
		//if(mp_init)
		{	
		int d;
		d = pcm_ioctl(PCM_GET_SPACE,0);
		//F("%s %d\n",__FUNCTION__,d); 
    //d = d % 65536;
		//d = d - (d & (0x10000 - 1));
		return d;
    
   }
   //mp_init = 1;
    //return ao_data.outburst;
    #endif
  
}

// plays 'len' bytes of 'data'
// it should round it down to outburst*n
// return: number of bytes played
#define	AFMT_U8			8
#define AFMT_S16_LE		16

void (*IS_3D_Play)(void *buffer,int size);
void Set3DPlay(void *p)
{
	IS_3D_Play=p;
}
static int play(void* data,int len,int flags){

// let libaf to do the conversion...
//#ifdef WORDS_BIGENDIAN
#if 0
	if (ao_data.format == AFMT_S16_LE) {
	  unsigned short *buffer = (unsigned short *) data;
	  register int i;
	  for(i = 0; i < len/2; ++i) {
	    buffer[i] = le2me_16(buffer[i]);
	  }
	}
#endif
	//printf("PCM: Writing chunk!\n");
	//F("%d\n",len);
        if(IS_3D_Play)
		IS_3D_Play(data,len);

	len = pcm_write(data,len);
  //F("%d\n",len);
	
	return len;
}

// return: delay in seconds between first and last sample in buffer
static float get_delay(void){

		
		int d;
		d = pcm_ioctl(PCM_GET_SPACE,0);
		//F("%d\n",d);
		d = ao_data.buffersize - d;
		//d = ao_data.buffersize - d;
		
		if( d <= 0)
			return 0.0;
    return ((float)d)/(float)ao_data.bps;
}


